==> tin-1.05.tar.gz: tin-1.05/kill.c <==
/*
 *  Project   : tin - a visual threaded usenet newsreader
 *  Module    : kill.c
 *  Author    : I.Lea
 *  Created   : 01-04-91
 *  Updated   : 30-09-91
 *  Release   : 1.0
 *  Notes     : kill articles
 *  Copyright : (c) Copyright 1991 by Iain Lea
 *	            You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

extern char index_file[LEN];

struct kill_t *killf;
int kill_num = 0;
int max_kill;

/*
 *  read_kill_file - read ~/.tin/kill file contents into kill array
 */

int read_kill_file ()
{
	char buf[LEN];
	FILE *fp;

	free_kill_array ();
	
	set_real_uid_gid ();

	if ((fp = fopen (killfile, "r")) != NULL) {
		kill_num=0;
		while (fgets (buf, sizeof buf, fp) != NULL) {
			if (buf[0] != '#') {
				if (kill_num == max_kill-1) {
					expand_kill ();
				}
				killf[kill_num].kill_type = (int) atoi (buf);
				if (fgets (buf, sizeof buf, fp) != NULL) {
					killf[kill_num].kill_group = (long) atol (buf);
				} else {	
					goto corrupt_killfile;
				}	
				switch (killf[kill_num].kill_type) {
					case KILL_SUBJ:
						if (fgets (buf, sizeof buf, fp) != NULL) {
							buf[strlen (buf)-1] = '\0';
							killf[kill_num].kill_subj = str_dup (buf);
						}
						break;
					case KILL_FROM:
						if (fgets (buf, sizeof buf, fp) != NULL) {
							buf[strlen (buf)-1] = '\0';
							killf[kill_num].kill_from = str_dup (buf);
						}
						break;
					case KILL_BOTH:
						if (fgets (buf, sizeof buf, fp) != NULL) {
							buf[strlen (buf)-1] = '\0';
							killf[kill_num].kill_subj = str_dup (buf);
						}
						if (fgets (buf, sizeof buf, fp) != NULL) {
							buf[strlen (buf)-1] = '\0';
							killf[kill_num].kill_from = str_dup (buf);
						}
						break;
					default:
						goto corrupt_killfile;
				}
				kill_num++;
			}
		}
		fclose (fp);
		set_tin_uid_gid ();
		return TRUE;		
	} else {
		set_tin_uid_gid ();
		return FALSE;
	}

corrupt_killfile:
	fclose (fp);
	killf[kill_num].kill_type = 0;
	error_message ("corrupt kill file %s", killfile);
	set_tin_uid_gid ();
	return FALSE;
}

/*
 *  write_kill_file - write kill strings to ~/.tin/kill
 */

void write_kill_file ()
{
	FILE *fp;
	int i;
	
	set_real_uid_gid ();

	if (kill_num && (fp = fopen (killfile, "w")) != NULL) {
		wait_message (txt_saving);
		fprintf (fp, "# 1st line  1=(Subject: only)  2=(From: only)  3=(Subject: & From:)\n");
		fprintf (fp, "# 2nd line  0=(kill on all newsgroups)  >0=(kill on specific newsgroup)\n");
		for (i=0 ; i < kill_num ; i++) {
			if (killf[i].kill_type && (killf[i].kill_subj || killf[i].kill_from)) {
				fprintf (fp, "#\n# kill description %d\n", i+1);
				fprintf (fp, "%d\n", killf[i].kill_type);
				fprintf (fp, "%ld\n", killf[i].kill_group);
				switch (killf[i].kill_type) {
					case KILL_SUBJ:
						fprintf (fp, "%s\n", killf[i].kill_subj);
						break;
					case KILL_FROM:
						fprintf (fp, "%s\n", killf[i].kill_from);
						break;
					case KILL_BOTH:
						fprintf (fp, "%s\n", killf[i].kill_subj);
						fprintf (fp, "%s\n", killf[i].kill_from);
						break;
				}
			}
		}
		fclose (fp);
		chmod (killfile, 0600);
	}
	set_tin_uid_gid ();
}

/*
 *  options menu so that the user can dynamically change parameters
 */
 
int kill_art_menu (group_name, index)
	char *group_name;
	int index;
{
	char text[LEN];
	char kill_from[LEN];
	char kill_subj[LEN];
	char kill_group[LEN];
	char ch_default = 's';
	char *str;	
	int ch;
	int counter;
	int killed = TRUE;
	int kill_from_ok = FALSE;
	int kill_subj_ok = FALSE;
	int kill_every_group;

#ifdef SIGTSTP
	void (*susp)();

	if (do_sigtstp) {
#ifdef POSIX_JOB_CONTROL
		sigemptyset (&kill_act.sa_mask);
		kill_act.sa_flags = SA_RESTART | SA_RESETHAND;
		kill_act.sa_handler = SIG_DFL;
		sigaction (SIGTSTP, &kill_act, &old_act);
		kill_act.sa_handler = SIG_IGN;
		sigaction (SIGTSTP, &kill_act, 0L);
#else
		susp = signal (SIGTSTP, SIG_DFL);
		signal (SIGTSTP, SIG_IGN);
#endif
	}
#endif
	
	sprintf (kill_group, "%s only", group_name);
	my_strncpy (text, arts[index].subject, COLS-35);
	sprintf (kill_subj, txt_kill_subject, text);
	my_strncpy (text, arts[index].from, COLS-35);
	sprintf (kill_from, txt_kill_from, text);
	text[0] = '\0';
	
	ClearScreen ();

	center_line (0, TRUE, txt_kill_menu);
	
	MoveCursor(INDEX_TOP, 0);
	printf ("%s\r\n\r\n", txt_kill_text);
	printf ("%s\r\n\r\n\r\n", txt_kill_text_type);
	printf ("%s\r\n\r\n", kill_subj);
	printf ("%s\r\n\r\n\r\n", kill_from);
	printf ("%s%s", txt_kill_group, kill_group);
	fflush(stdout);

	show_menu_help (txt_help_kill_text);
	if (! parse_menu_string (INDEX_TOP, (int) strlen (txt_kill_text), text)) {
		return FALSE;
	}

	if (text[0]) {
		show_menu_help (txt_help_kill_text_type);
		counter = 1;
		MoveCursor (INDEX_TOP+2, (int) strlen (txt_kill_text_type));
		str = "Subject: line only    ";
		printf ("%s", str);
		fflush(stdout);
		do {
			MoveCursor (INDEX_TOP+2, (int) strlen (txt_kill_text_type));
			if ((ch	= ReadCh()) == ' ') {
				counter++;
				if (counter == KILL_BOTH+1) {
					counter = KILL_SUBJ;
				}
				switch (counter) {
					case KILL_SUBJ:
						str = "Subject: line only    ";
						break;
					case KILL_FROM:
						str = "From: line only       ";
						break;
					case KILL_BOTH:
						str = "Subject: & From: lines";
						break;
				}
				printf ("%s", str);
				fflush(stdout);
			}
		} while (ch != CR && ch != ESC);
		if (ch == ESC) {
			return FALSE;
		}
	}

	if (! text[0]) {
		show_menu_help (txt_help_kill_subject);
		kill_subj_ok = prompt_yn (INDEX_TOP+5, kill_subj, 'y');

		show_menu_help (txt_help_kill_from);
		kill_from_ok = prompt_yn (INDEX_TOP+7, kill_from, 'n');
	}

	if (text[0] || kill_subj_ok || kill_from_ok) {
		show_menu_help (txt_help_kill_group);
		kill_every_group = FALSE;
		MoveCursor (INDEX_TOP+10, (int) strlen (txt_kill_group));
		str = kill_group;
		printf ("%s", str);
		fflush(stdout);
		do {
			MoveCursor (INDEX_TOP+10, (int) strlen (txt_kill_group));
			if ((ch	= ReadCh()) == ' ') {
				kill_every_group = !kill_every_group;
				if (kill_every_group) {
					str = "All groups";
				} else {
					str = kill_group;
				}
				CleartoEOLN (); 
				printf ("%s", str);
				fflush(stdout);
			}
		} while (ch != CR && ch != ESC);
		if (ch == ESC) {
			return FALSE;
		}
	}

	while (1) {
		do {
			sprintf (msg, "%s%c", txt_abort_edit_save_killfile, ch_default);
			wait_message (msg);
			MoveCursor(LINES, (int) strlen (txt_abort_edit_save_killfile));
			if ((ch = ReadCh()) == CR)
				ch = ch_default;
		} while (ch != ESC && ch != 'a' && ch != 'e' && ch != 's');
		switch (ch) {
			case 'e':
				start_line_offset = 2;
				invoke_editor (killfile);
				untag_all_articles ();
				read_kill_file ();
				reload_index_file (group_name, FALSE);
				killed = TRUE;
				goto kill_done;

			case 'a':
			case ESC:
				killed = FALSE;
				goto kill_done;
			
			case 's':
				if (kill_num > max_kill-1) {
					expand_kill ();
				}
				if (text[0]) {
					switch (counter) {
						case KILL_SUBJ:
							killf[kill_num].kill_subj = str_dup (text);
							break;
						case KILL_FROM:
							killf[kill_num].kill_from = str_dup (text);
							break;
						case KILL_BOTH:
							killf[kill_num].kill_subj = str_dup (text);
							killf[kill_num].kill_from = str_dup (text);
							break;
					}
					killf[kill_num].kill_type = counter;
					if (kill_every_group) {
						killf[kill_num].kill_group= 0L;
					} else {
						killf[kill_num].kill_group= hash_s (group_name);
					}
					kill_num++;
				} else {
					if (kill_subj_ok) {
						killf[kill_num].kill_type = KILL_SUBJ;
						killf[kill_num].kill_subj = str_dup (arts[index].subject);
					}
					if (kill_from_ok) {
						killf[kill_num].kill_type |= KILL_FROM;
						killf[kill_num].kill_from = str_dup (arts[index].from);
					}
					if (killf[kill_num].kill_type) {		
						if (kill_every_group) {
							killf[kill_num].kill_group= 0L;
						} else {
							killf[kill_num].kill_group= hash_s (group_name);
						}
						kill_num++;
					}
				}
				write_kill_file ();

kill_done:

#ifdef SIGTSTP
				if (do_sigtstp) {
#ifdef POSIX_JOB_CONTROL
					sigemptyset (&kill_act.sa_mask);
					kill_act.sa_flags = SA_RESTART | SA_RESETHAND;
					kill_act.sa_handler = SIG_IGN;
					sigaction (SIGTSTP, &old_act, 0L);
#else
					signal (SIGTSTP, susp);
#endif
				}
#endif
				return (killed);
		}	
	}
	/* NOTREACHED */
}


int untag_all_articles ()
{
	int untagged = FALSE;
	register int i;

	for (i=0 ; i < top ; i++) {
		if (arts[i].tagged) {
			arts[i].tagged = FALSE;
			untagged = TRUE;
		}
	}
	num_of_tagged_files = 0;

	return (untagged);
}


int kill_any_articles (group)
	char *group;
{
	int killed = FALSE;
	int run_ok = FALSE;
	long group_hash;
	register int i, j;

	if (! kill_articles) {
		return killed;
	}

	if (kill_num) {
		group_hash = hash_s (group);
		for (i=0 ; i < kill_num ; i++) {
				if (killf[i].kill_group == 0L ||
					killf[i].kill_group == group_hash) {
					run_ok = TRUE;	
				}
		}
		if (! run_ok) {
			return (killed);
		}
		if (debug && ! update) {
			wait_message (txt_killing_arts);
		}
		for (i=0 ; i < top ; i++) {
			for (j=0 ; j < kill_num && ! arts[i].tagged ; j++) {
				if (killf[j].kill_group == 0L ||
					killf[j].kill_group == group_hash) {
					switch (killf[j].kill_type) {
						case KILL_SUBJ:
#ifdef DONT_USE_REGEX 
							if (str_str (arts[i].subject, killf[j].kill_subj) != 0) {
#else		
							if (wildmat (arts[i].subject, killf[j].kill_subj)) {
#endif		

								arts[i].tagged = TRUE;
								killed = TRUE;
							}
							break;
						case KILL_FROM:
#ifdef DONT_USE_REGEX 
							if (str_str (arts[i].from, killf[j].kill_from) != 0) {
#else		
							if (wildmat (arts[i].from, killf[j].kill_from)) {
#endif		
								arts[i].tagged = TRUE;
								killed = TRUE;
							}
							break;
						case KILL_BOTH:
#ifdef DONT_USE_REGEX 
							if (str_str (arts[i].subject, killf[j].kill_subj) != 0) {
#else		
							if (wildmat (arts[i].subject, killf[j].kill_subj)) {
#endif		
								arts[i].tagged = TRUE;
								killed = TRUE;
							}
#ifdef DONT_USE_REGEX 
							if (str_str (arts[i].from, killf[j].kill_from) != 0) {
#else		
							if (wildmat (arts[i].from, killf[j].kill_from)) {
#endif		
								arts[i].tagged = TRUE;
								killed = TRUE;
							}
							break;
					}
				}
			}
		}
	}
	return (killed);
}

==> tin-1.10.tar.gz: tin-1.1/kill.c <==
/*
 *  Project   : tin - a threaded Netnews reader
 *  Module    : kill.c
 *  Author    : I.Lea
 *  Created   : 01-04-91
 *  Updated   : 31-01-92
 *  Notes     : kill articles
 *  Copyright : (c) Copyright 1991-92 by Iain Lea
 *              You may  freely  copy or  redistribute  this software,
 *              so  long as there is no profit made from its use, sale
 *              trade or  reproduction.  You may not change this copy-
 *              right notice, and it must be included in any copy made
 */

#include	"tin.h"

extern char index_file[LEN];

struct kill_t *killf;
int kill_num = 0;
int max_kill;

/*
 *  read_kill_file - read ~/.tin/kill file contents into kill array
 */

int read_kill_file ()
{
	char buf[LEN];
	FILE *fp;

	free_kill_array ();
	
	set_real_uid_gid ();

	if ((fp = fopen (killfile, "r")) != NULL) {
		kill_num=0;
		while (fgets (buf, sizeof buf, fp) != NULL) {
			if (buf[0] != '#') {
				if (kill_num == max_kill-1) {
					expand_kill ();
				}
				killf[kill_num].kill_type = (int) atoi (buf);
				if (fgets (buf, sizeof buf, fp) != NULL) {
					killf[kill_num].kill_group = (long) atol (buf);
				} else {	
					goto corrupt_killfile;
				}	
				switch (killf[kill_num].kill_type) {
					case KILL_SUBJ:
						if (fgets (buf, sizeof buf, fp) != NULL) {
							buf[strlen (buf)-1] = '\0';
							killf[kill_num].kill_subj = str_dup (buf);
						}
						break;
					case KILL_FROM:
						if (fgets (buf, sizeof buf, fp) != NULL) {
							buf[strlen (buf)-1] = '\0';
							killf[kill_num].kill_from = str_dup (buf);
						}
						break;
					case KILL_BOTH:
						if (fgets (buf, sizeof buf, fp) != NULL) {
							buf[strlen (buf)-1] = '\0';
							killf[kill_num].kill_subj = str_dup (buf);
						}
						if (fgets (buf, sizeof buf, fp) != NULL) {
							buf[strlen (buf)-1] = '\0';
							killf[kill_num].kill_from = str_dup (buf);
						}
						break;
					default:
						goto corrupt_killfile;
				}
				kill_num++;
			}
		}
		fclose (fp);
		set_tin_uid_gid ();
		return TRUE;		
	} else {
		set_tin_uid_gid ();
		return FALSE;
	}

corrupt_killfile:
	fclose (fp);
	killf[kill_num].kill_type = 0;
	error_message ("corrupt kill file %s", killfile);
	set_tin_uid_gid ();
	return FALSE;
}

/*
 *  write_kill_file - write kill strings to ~/.tin/kill
 */

void write_kill_file ()
{
	FILE *fp;
	int i;
	
	set_real_uid_gid ();

	if (kill_num && (fp = fopen (killfile, "w")) != NULL) {
		wait_message (txt_saving);
		fprintf (fp, "# 1st line  1=(Subject: only)  2=(From: only)  3=(Subject: & From:)\n");
		fprintf (fp, "# 2nd line  0=(kill on all newsgroups)  >0=(kill on specific newsgroup)\n");
		for (i=0 ; i < kill_num ; i++) {
			if (killf[i].kill_type && (killf[i].kill_subj || killf[i].kill_from)) {
				fprintf (fp, "#\n# kill description %d\n", i+1);
				fprintf (fp, "%d\n", killf[i].kill_type);
				fprintf (fp, "%ld\n", killf[i].kill_group);
				switch (killf[i].kill_type) {
					case KILL_SUBJ:
						fprintf (fp, "%s\n", killf[i].kill_subj);
						break;
					case KILL_FROM:
						fprintf (fp, "%s\n", killf[i].kill_from);
						break;
					case KILL_BOTH:
						fprintf (fp, "%s\n", killf[i].kill_subj);
						fprintf (fp, "%s\n", killf[i].kill_from);
						break;
				}
			}
		}
		fclose (fp);
		chmod (killfile, 0600);
	}
	set_tin_uid_gid ();
}

/*
 *  options menu so that the user can dynamically change parameters
 */
 
int kill_art_menu (group_name, index)
	char *group_name;
	int index;
{
	char text[LEN];
	char kill_from[LEN];
	char kill_subj[LEN];
	char kill_group[LEN];
	char ch_default = 's';
	char *str;	
	int ch;
	int counter;
	int killed = TRUE;
	int kill_from_ok = FALSE;
	int kill_subj_ok = FALSE;
	int kill_every_group;

#ifdef SIGTSTP
	SIGTYPE (*susp)();

	if (do_sigtstp) {
#ifdef POSIX_JOB_CONTROL
		sigemptyset (&kill_act.sa_mask);
		kill_act.sa_flags = SA_RESTART | SA_RESETHAND;
		kill_act.sa_handler = SIG_DFL;
		sigaction (SIGTSTP, &kill_act, &old_act);
		kill_act.sa_handler = SIG_IGN;
		sigaction (SIGTSTP, &kill_act, 0L);
#else
		susp = signal (SIGTSTP, SIG_DFL);
		signal (SIGTSTP, SIG_IGN);
#endif
	}
#endif
	
	sprintf (kill_group, "%s only", group_name);
	my_strncpy (text, arts[index].subject, COLS-35);
	sprintf (kill_subj, txt_kill_subject, text);
	if (arts[index].name != (char *) 0) {
		sprintf (msg, "%s (%s)", arts[index].from, arts[index].name);
		my_strncpy (text, msg, COLS-35);
	} else {
		my_strncpy (text, arts[index].from, COLS-35);
	}
	sprintf (kill_from, txt_kill_from, text);
	text[0] = '\0';
	
	ClearScreen ();

	center_line (0, TRUE, txt_kill_menu);
	
	MoveCursor(INDEX_TOP, 0);
	printf ("%s\r\n\r\n", txt_kill_text);
	printf ("%s\r\n\r\n\r\n", txt_kill_text_type);
	printf ("%s\r\n\r\n", kill_subj);
	printf ("%s\r\n\r\n\r\n", kill_from);
	printf ("%s%s", txt_kill_group, kill_group);
	fflush(stdout);

	show_menu_help (txt_help_kill_text);
	if (! prompt_menu_string (INDEX_TOP, (int) strlen (txt_kill_text), text)) {
		return FALSE;
	}

	if (text[0]) {
		show_menu_help (txt_help_kill_text_type);
		counter = 1;
		MoveCursor (INDEX_TOP+2, (int) strlen (txt_kill_text_type));
		str = "Subject: line only    ";
		printf ("%s", str);
		fflush(stdout);
		do {
			MoveCursor (INDEX_TOP+2, (int) strlen (txt_kill_text_type));
			if ((ch	= ReadCh()) == ' ') {
				counter++;
				if (counter == KILL_BOTH+1) {
					counter = KILL_SUBJ;
				}
				switch (counter) {
					case KILL_SUBJ:
						str = "Subject: line only    ";
						break;
					case KILL_FROM:
						str = "From: line only       ";
						break;
					case KILL_BOTH:
						str = "Subject: & From: lines";
						break;
				}
				printf ("%s", str);
				fflush(stdout);
			}
		} while (ch != CR && ch != ESC);
		if (ch == ESC) {
			return FALSE;
		}
	}

	if (! text[0]) {
		show_menu_help (txt_help_kill_subject);
		kill_subj_ok = prompt_yn (INDEX_TOP+5, kill_subj, 'y');

		show_menu_help (txt_help_kill_from);
		kill_from_ok = prompt_yn (INDEX_TOP+7, kill_from, 'n');
	}

	if (text[0] || kill_subj_ok || kill_from_ok) {
		show_menu_help (txt_help_kill_group);
		kill_every_group = FALSE;
		MoveCursor (INDEX_TOP+10, (int) strlen (txt_kill_group));
		str = kill_group;
		printf ("%s", str);
		fflush (stdout);
		do {
			MoveCursor (INDEX_TOP+10, (int) strlen (txt_kill_group));
			if ((ch	= ReadCh()) == ' ') {
				kill_every_group = !kill_every_group;
				if (kill_every_group) {
					str = "All groups";
				} else {
					str = kill_group;
				}
				CleartoEOLN (); 
				printf ("%s", str);
				fflush(stdout);
			}
		} while (ch != CR && ch != ESC);
		if (ch == ESC) {
			return FALSE;
		}
	}

	while (1) {
		do {
			sprintf (msg, "%s%c", txt_abort_edit_save_killfile, ch_default);
			wait_message (msg);
			MoveCursor(LINES, (int) strlen (txt_abort_edit_save_killfile));
			if ((ch = ReadCh()) == CR)
				ch = ch_default;
		} while (ch != ESC && ch != 'a' && ch != 'e' && ch != 's');
		switch (ch) {
			case 'e':
				start_line_offset = 2;
				invoke_editor (killfile);
				unkill_all_articles ();
				read_kill_file ();
				reload_index_file (group_name, FALSE);
				killed = TRUE;
				goto kill_done;

			case 'a':
			case ESC:
				killed = FALSE;
				goto kill_done;
			
			case 's':
				if (kill_num > max_kill-1) {
					expand_kill ();
				}
				if (text[0]) {
					switch (counter) {
						case KILL_SUBJ:
							killf[kill_num].kill_subj = str_dup (text);
							break;
						case KILL_FROM:
							killf[kill_num].kill_from = str_dup (text);
							break;
						case KILL_BOTH:
							killf[kill_num].kill_subj = str_dup (text);
							killf[kill_num].kill_from = str_dup (text);
							break;
					}
					killf[kill_num].kill_type = counter;
					if (kill_every_group) {
						killf[kill_num].kill_group= 0L;
					} else {
						killf[kill_num].kill_group= hash_s (group_name);
					}
					kill_num++;
				} else {
					if (kill_subj_ok) {
						killf[kill_num].kill_type = KILL_SUBJ;
						killf[kill_num].kill_subj = str_dup (arts[index].subject);
					}
					if (kill_from_ok) {
						killf[kill_num].kill_type |= KILL_FROM;
						if (arts[index].name != (char *) 0) {
							sprintf (msg, "%s (%s)", arts[index].from, arts[index].name);
						} else {
							strcpy (msg, arts[index].from);
						}
						killf[kill_num].kill_from = str_dup (msg);
					}
					if (killf[kill_num].kill_type) {		
						if (kill_every_group) {
							killf[kill_num].kill_group= 0L;
						} else {
							killf[kill_num].kill_group= hash_s (group_name);
						}
						kill_num++;
					}
				}
				write_kill_file ();

kill_done:

#ifdef SIGTSTP
				if (do_sigtstp) {
#ifdef POSIX_JOB_CONTROL
					sigemptyset (&kill_act.sa_mask);
					kill_act.sa_flags = SA_RESTART | SA_RESETHAND;
					kill_act.sa_handler = SIG_IGN;
					sigaction (SIGTSTP, &old_act, 0L);
#else
					signal (SIGTSTP, susp);
#endif
				}
#endif
				return (killed);
		}	
	}
	/* NOTREACHED */
}


int unkill_all_articles ()
{
	int unkilled = FALSE;
	register int i;

	for (i=0 ; i < top ; i++) {
		if (arts[i].killed) {
			arts[i].killed = FALSE;
			unkilled = TRUE;
		}
	}
	num_of_killed_files = 0;

	return (unkilled);
}


int kill_any_articles (group)
	char *group;
{
	int killed = FALSE;
	int run_ok = FALSE;
	long group_hash;
	register int i, j;

	if (! kill_articles) {
		return killed;
	}

	if (kill_num) {
		group_hash = hash_s (group);
		for (i=0 ; i < kill_num ; i++) {
				if (killf[i].kill_group == 0L ||
					killf[i].kill_group == group_hash) {
					run_ok = TRUE;	
				}
		}
		if (! run_ok) {
			return (killed);
		}
		if (debug && ! update) {
			wait_message (txt_killing_arts);
		}
		for (i=0 ; i < top ; i++) {
			for (j=0 ; j < kill_num && ! arts[i].tagged ; j++) {
				if (killf[j].kill_group == 0L ||
					killf[j].kill_group == group_hash) {
					switch (killf[j].kill_type) {
						case KILL_SUBJ:
#ifdef NO_REGEX 
							if (str_str (arts[i].subject, killf[j].kill_subj,
										strlen (killf[j].kill_subj)) != 0) {
#else		
							if (wildmat (arts[i].subject, killf[j].kill_subj)) {
#endif		

								arts[i].tagged = TRUE;
								killed = TRUE;
							}
							break;
						case KILL_FROM:
							if (arts[i].name != (char *) 0) {
								sprintf (msg, "%s (%s)", arts[i].from, arts[i].name);
							} else {
								strcpy (msg, arts[i].from);
							}
#ifdef NO_REGEX 
							if (str_str (msg, killf[j].kill_from
										strlen (killf[j].kill_from)) != 0) {
#else		
							if (wildmat (msg, killf[j].kill_from)) {
#endif		
								arts[i].tagged = TRUE;
								killed = TRUE;
							}
							break;
						case KILL_BOTH:
#ifdef NO_REGEX 
							if (str_str (arts[i].subject, killf[j].kill_subj,
										strlen (killf[j].kill_subj)) != 0) {
#else		
							if (wildmat (arts[i].subject, killf[j].kill_subj)) {
#endif		
								arts[i].tagged = TRUE;
								killed = TRUE;
							}
							if (arts[i].name != (char *) 0) {
								sprintf (msg, "%s (%s)", arts[i].from, arts[i].name);
							} else {
								strcpy (msg, arts[i].from);
							}
#ifdef NO_REGEX
							if (str_str (msg, killf[j].kill_from,
										strlen (killf[j].kill_from)) != 0) {
#else		
							if (wildmat (msg, killf[j].kill_from)) {
#endif		
								arts[i].tagged = TRUE;
								killed = TRUE;
							}
							break;
					}
				}
			}
		}
	}
	return (killed);
}
